-----------------------------------------------------------------------------------------------
--
--   SLR Converter Pulse Controller Module
--   Eirik Taylor, 04.06.2022
--   
--   
--   Synchronizes output pulses with an incoming enable signal,
--   and ensures drive is alternated between each output, even after
--   enable is used. Application is high speed interruption of drive
--   pulses.
--
-----------------------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity slr_pulse_controller is
port (
		drive_clk		: in STD_LOGIC;
		enable		   : in STD_LOGIC;
		gateA_drv		: out STD_LOGIC;
		gateB_drv		: out STD_LOGIC;
		gateA_en			: out STD_LOGIC;
		gateB_en			: out STD_LOGIC
		--debug0			: out STD_LOGIC;
		--debug1			: out STD_LOGIC;
		--debug2			: out STD_LOGIC
	);
end slr_pulse_controller;

architecture Behavioral of slr_pulse_controller is
	
-- Dummy signal
	signal rst: STD_LOGIC;

-- Interrupt signals
	signal enable_syncd: STD_LOGIC;
	
-- Last interrupted side memory
	signal pulse_active_side: STD_LOGIC;
	signal lastLatched_A : STD_LOGIC;
	signal lastLatched_B : STD_LOGIC;

-- Output signals
	signal gateEnable_A: STD_LOGIC;
	signal gateEnable_B: STD_LOGIC;
	signal activeSide_A: STD_LOGIC;
	signal activeSide_B: STD_LOGIC;

begin

-- Debugging pins
--debug0 <= drive_clk;
--debug1 <= rst;
--debug2 <= enable_syncd;

-- Dummy reset signal
rst <= '1';

-- Driver enable outputs are always high. Enable handled internally
-- to avoid glitches from delays.
gateA_en <= '1';
gateB_en <= '1';
gateA_drv <= drive_clk and gateEnable_A;
gateB_drv <= drive_clk and gateEnable_B;

-- Only enable a side if it's it's turn, and the external enable signal is high
gateEnable_A <= activeSide_A and enable_syncd;
gateEnable_B <= activeSide_B and enable_syncd;

-- Pulse any time the active side changes
pulse_active_side <= (lastLatched_A and activeSide_A) or
					 (lastLatched_B and activeSide_B);

-- Connect the T-latches
	drive_leg_toggle_t_latch : entity work.t_latch
	port map (
		rst => rst,
		clk => drive_clk,
		Q => activeSide_A,
		nQ => activeSide_B
	);
	
	last_interrupted_side_toggle_t_latch : entity work.t_latch
	port map (
		rst => rst,
		clk => enable_syncd,
		Q => lastLatched_A,
		nQ => lastLatched_B
	);
	
-- Synchronize the interruption signal with the clock, and with
-- the last interrupted side
	enableSyncProcess : process (pulse_active_side, rst)
	begin
		if (rst = '0') then 
			enable_syncd <= '0'; 
		elsif (rising_edge(pulse_active_side)) then 
			enable_syncd <= enable;
		end if;
	end process;

end Behavioral;

